import { Plugin } from 'foundation-sites/js/foundation.core.plugin'
import $ from 'jquery'

/**
 * User agent detection module.
 *
 * Show or hide elements depending on user agent data. Possible options match
 * the properties of NavigatorUAdata. If the browser does not yet support the API,
 * there is a fallback to the user agent string.
 *
 * For now, only "mobile" and "platform" are implemented.
 *
 * @module foundation.userAgent
 */

class UserAgent extends Plugin {
  /**
   * Creates a new instance.
   * @class
   * @name UserAgent
   * @fires UserAgent#init
   * @param {JQuery} element - jQuery object to show or hide.
   * @param {object} options - Overrides to the default plugin settings.
   */
  _setup (element, options = {}) {
    this.$element = element
    this.className = 'UserAgent'
    // defaults, fallbacks
    this.options = $.extend(true, {}, UserAgent.defaults, this.$element.data(), options)
    this.uaData = {}

    this._log('setup', this)

    // Only do anything in an actual browser.
    if (!navigator) {
      this._log('no navigator', this)
      return
    }

    this._init()
  }

  /**
   * Read user agent data and take action accordingly.
   * @function
   * @private
   */
  _init () {
    if (typeof navigator.userAgentData?.toJSON === 'function') {
      this.uaData = navigator.userAgentData.toJSON()
    }
    this._log('init', this)

    if (this._checkMobile() || this._checkPlatform()) {
      this._applyAction()
    }
  }

  /**
   * Check mobile device.
   * @function
   * @private
   */
  _checkMobile () {
    if (typeof this.options.mobile !== 'boolean') {
      return false
    }
    if (this.uaData.length) {
      return this.uaData.mobile === this.options.mobile
    }
    return /mobi/i.test(navigator.userAgent) === this.options.mobile
  }

  /**
   * Check platform name.
   * @function
   * @private
   */
  _checkPlatform () {
    if (typeof this.options.platform !== 'string') {
      return false
    }
    if (this.uaData.length) {
      return this.options.platform.toLower().includes(this.uaData.platform.toLower())
    }
    return this.options.platform.trim().split(' ').some((platform) => {
      return (new RegExp(platform, 'i')).test(navigator.userAgent)
    })
  }

  /**
   * Apply the appropriate action.
   * @function
   * @private
   */
  _applyAction () {
    const hidden = this.$element.hasClass(this.options.hiddenClass)
    if (hidden && this.options.action === 'show') {
      this.show()
    } else if (!hidden && this.options.action === 'hide') {
      this.hide()
    }
  }

  /*
   * Show element.
   * @function
   * @fires UserAgent#shown
   */
  show () {
    this._log('show', this)
    this.$element.removeClass(this.options.hiddenClass)
    this._log(this.options.hiddenClass, 'hidden class')

    /**
     * Fires when the target is shown.
     * @event UserAgent#shown
     */
    this.$element.trigger('shown.zf.userAgent')
  }

  /**
   * Hide element.
   * @function
   * @fires UserAgent#hidden
   */
  hide () {
    this._log('hide', this)
    this.$element.addClass(this.options.hiddenClass)

    /**
     * Fires when the target is hidden.
     * @event UserAgent#hidden
     */
    this.$element.trigger('hidden.zf.userAgent')
  }

  /**
   * Destroys the UserAgent plugin.
   * @function
   * @private
   */
  _destroy () {
    this._log('destroy', this)
  }

  /**
   * Logs debug output to the console.
   * @function
   * @private
   */
  _log () {
    if (this.options.debug) {
      console.debug('[UserAgent]', ...arguments)
    }
  }
}

/**
 * Default settings for plugin.
 */
UserAgent.defaults = {
  /**
   * The action to apply to the element: 'show' or 'hide'.
   * @option
   * @type {string}
   * @default 'show'
   */
  action: 'show',

  /**
   * Check for a mobile device.
   * @option
   * @type {boolean|null}
   * @default null
   */
  mobile: null,

  /**
   * Check for a specific platform (or any of list separated by space).
   * @option
   * @type {string|null}
   * @default null
   */
  platform: null,

  /**
   * The class set on the target while it’s hidden.
   *
   * If this is already there on page load, it will hide the target initially.
   * @option
   * @type {string}
   * @default 'hide'
   */
  hiddenClass: 'hide',

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false,
}

export { UserAgent }
